/* 
 *    fandango, a GL rendered, python scripted drawing engine
 *                  Copyright (C) 2002, 2003 by
 *
 *      Jaime Soffer             (jsoffer@soffernet.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "entitymarshall.hpp"

const unsigned int ENTITIES_IN_PAGE = 4096;

// [FIXME]: synchronize with iCore.cc
const unsigned int NUM_LAYERS = 256;
const unsigned int HIGHLIGHT = NUM_LAYERS - 3;

const float REALLYBIGFLOAT = 4294967296.0f; // 2^32

entityMarshall::entityMarshall(void){

  colors_array = NULL;

  blending = false;
  texture = false;
  lighting = true;

  select = false;
  selected_entity = 0;
  memset(selected_entity_array, 0, 4*sizeof(unsigned int));

  snap = false;
  // Even when this is a float, it represents distance in pixels
  snap_distance = 5.f;

  // Reminder: these are arrays, but may become pointers
  //  snap_vertex = NULL; 
  //  active_intersection = NULL;
  memset(snap_vertex, 0, sizeof(float)*3);
  memset(active_intersection, 0, sizeof(float)*3);

  points_count = 0;

  // This is the value that's being used carelessly. Its current value
  // is the *next* entity that will be created, entities_count-1 is being
  // used as the current entity but that's wrong because there are four
  // active memory pages at a given moment, each with its current entity
  //                                (FIXME)
  entities_count = 0;
  faces_count = 0;
  lines_count = 0;

  scale_over_viewport = 1.0f;

  cards_array = new entityCard [ENTITIES_IN_PAGE];
  memset(cards_array, 0, sizeof(entityCard)*ENTITIES_IN_PAGE);

  // WARNING: TESTING
  //  memset(active_vertex, 0, sizeof(float)*3);
  active_vertex = NULL;

  vertex_intersection_entity = 0; // VALGRINDED
  vertex_selected_entity = 0; // VALGRINDED

  duplication = false;
}

entityMarshall::~entityMarshall(void){
  if(cards_array != NULL){
    delete [] cards_array; cards_array = NULL;
  }
}

bool entityMarshall::doubleBeginEntityTest(void){
  return duplication;
}

void entityMarshall::doubleBeginEntityTest(bool b){
  duplication = b;
}

void entityMarshall::drawActiveVertex(unsigned int d){
 
  // Dancing with the death...
  // Don't forget to initialize to NULL!
  if(active_vertex != NULL){ 

    float size = (float) d * scale_over_viewport;

    glDepthFunc(GL_ALWAYS);
    glColor4f(1.f, 1.f, 0.f, 0.4f);
    glBegin(GL_LINE_LOOP);
    glVertex3f(active_vertex[0]-size, active_vertex[1]-size, 
	       active_vertex[2]);
    glVertex3f(active_vertex[0]+size, active_vertex[1]-size, 
	       active_vertex[2]);
    glVertex3f(active_vertex[0]+size, active_vertex[1]+size, 
	       active_vertex[2]);
    glVertex3f(active_vertex[0]-size, active_vertex[1]+size, 
	       active_vertex[2]);
    glEnd();

    glDepthFunc(GL_LESS);
  }
  
}

// Ancillary
void entityMarshall::drawActiveInt(unsigned int d){

  float size = (float) d * scale_over_viewport;

  // FIX (Do not light the ancillary)
  glDisable(GL_LIGHTING);
  
  glDepthFunc(GL_ALWAYS);
  glColor4f(1.f, 1.f, 0.f, 0.4f);
  
  glBegin(GL_LINES);
  glVertex3f(active_intersection[0]-size, active_intersection[1]-size, 
	     active_intersection[2]);
  glVertex3f(active_intersection[0]+size, active_intersection[1]+size, 
	     active_intersection[2]);
  glVertex3f(active_intersection[0]+size, active_intersection[1]-size, 
	     active_intersection[2]);
  glVertex3f(active_intersection[0]-size, active_intersection[1]+size, 
	     active_intersection[2]);
  glEnd();

  glDepthFunc(GL_LESS);

  // FIX
  applyLighting(true);

}

// Ancillary
void entityMarshall::drawBoundbox(unsigned int n, float mag){

  glColor4f(0.f, 0.f, mag, 1.0f);
  
  glBegin(GL_LINE_LOOP);

  glVertex2f( (cards_array + n)->left,
	      (cards_array + n)->top);
  glVertex2f( (cards_array + n)->right,
	      (cards_array + n)->top);
  glVertex2f( (cards_array + n)->right,
	      (cards_array + n)->bottom);
  glVertex2f( (cards_array + n)->left,
	      (cards_array + n)->bottom);
  glEnd();
}

// OPTIMIZE: applyLighting, for example, may not need to be called so often
// Drawing: draws entity number n
void entityMarshall::draw(unsigned int n) {
  // Currently holds a fixed number of entities
  if ((n >= ENTITIES_IN_PAGE) || (n >= entities_count)) return;

  // if((cards_array + n)->visible)
  if(true){
    glVertexPointer(3, GL_FLOAT, 0, (cards_array + n)->data);
    
    if(((cards_array + n)->gltype == GL_TRIANGLES) ||
       ((cards_array + n)->gltype == GL_TRIANGLE_STRIP)
       ){
      applyLighting(true);
      applyTexture(true);

      // FIXME: hack to protect a pointer; possibly slow
      glEnableClientState(GL_NORMAL_ARRAY);
      glEnableClientState(GL_TEXTURE_COORD_ARRAY);

      glNormalPointer(GL_FLOAT, 0, (cards_array + n)->normals);
      glTexCoordPointer(2, GL_FLOAT, 0, (cards_array + n)->texture);
    } else {
      // FIXME: hack to protect a pointer; possibly slow
      glDisableClientState(GL_NORMAL_ARRAY);
      glDisableClientState(GL_TEXTURE_COORD_ARRAY);

      glDisable(GL_LIGHTING);
      glDisable(GL_TEXTURE_2D);
    }

    int i = -1;
    for (int j = 0; j < 3; ++j){
      if (n == selected_entity_array[j]){
	i = j; // FIXME: hack
	break;
      }
    }

    // The entity is on the selection queue
    if (select && (i >= 0)){ 
      drawBoundbox(n, (1.0/(i+1.0)));
      if(i == 0){
	glColor4fv(colors_array + (4*HIGHLIGHT));
      }
    } 

    // Not the currently selected entity
    if(!select || (i != 0)){
      glColor4fv((cards_array + n)->color);
    }

    // Current point of the segfault

    glDrawArrays((cards_array + n)->gltype, 
		 0,
		 (cards_array + n)->size);

  
    if(n == selected_entity_array[0] && n == vertex_selected_entity){
      drawActiveVertex(3);
    }
    
    if(n == vertex_intersection_entity){
      drawActiveInt(5);
    }
    
  }


}

void entityMarshall::setActiveVertex(unsigned int i, float* coordinates){
  //  memcpy(active_vertex, coordinates, sizeof(float)*3);
  vertex_selected_entity = i;
  active_vertex = coordinates; // WARNING: TESTING
}

// Implementation of idorumath::findLinesIntersection, 
// tries to find the closer intersection to a click.
// I think it traverses twice each intersection...
// Currently too slow. [OPTIMIZE] priority.

// Optimization path: reducing the calculations done on each point is 
// not producing noticeable results. The right way is to reduce the number
// of points the function processes, perharps through regionality.

// Optimization done: it is only called if on a rectangle delimited
// close to the click

int entityMarshall::findNearestIntersection(float x, float y, float z){

  using idorumath::findLinesIntersection;
  using idorumath::calculateSqrdDistancePP;

  bool found = false;

  bool response = false;
  float distance = 0.f;
  float tmpdistance = REALLYBIGFLOAT;
  float intersect[3];

  for(unsigned int e1 = 0; e1 < entities_count; ++e1){
    // This theorically would work, but doesn't.
    // for(unsigned int e2 = e1; e2 < entities_count; ++e2){
    // for(unsigned int e2 = e1; e2 < e1; ++e2){
    for(unsigned int e2 = 0; e2 < entities_count; ++e2){

      // Find out if the boundboxes intersect.
      // It is done twice because when they are nested
      // the algorithm fails.
      // Extended complexity (and in going to get worse) FIXME
      if(pointinbox(x, y, z, e1, e2)){
	// && boxboxintersection(e1, e2) || boxboxintersection(e2, e1)){
	for(unsigned int el1 = 0; el1 < (cards_array + e1)->size; ++el1){
	  for(unsigned int el2 = 0; el2 < (cards_array + e2)->size; ++el2){
	    const GLenum e1_type = (cards_array + e1)->gltype;
	    const GLenum e2_type = (cards_array + e2)->gltype;

	    // (x == GL_LINES) type comparisons are repeated needlessly
	    if(((e1_type == GL_LINES) && 
		(e2_type == GL_LINES) &&
		(!(el1%2)) &&
		(!(el2%2))
		) ||
	       ((e1_type == GL_LINES) && 
		(e2_type == GL_LINE_STRIP) &&
		// Is it the beginning of a line? Don't process intermissions
		(!(el1%2)) &&
		// trim the core smashing
		(el2 < ((cards_array + e2)->size - 1))
		) ||
	       ((e1_type == GL_LINE_STRIP) && 
		(e2_type == GL_LINES) &&
		(!(el2%2)) &&
		(el1 < ((cards_array + e1)->size - 1))
		) ||
	       ((e1_type == GL_LINE_STRIP) && 
		(e2_type == GL_LINE_STRIP) &&
		(el1 < ((cards_array + e1)->size - 1)) &&
		(el2 < ((cards_array + e2)->size - 1))
		)){
	      // Silly: calculates ALL the scene's intersections
	      // By this approach even a table would be better
	      response = 
		findLinesIntersection((cards_array + e1)->data + el1*3,
				      (cards_array + e2)->data + el2*3,
				      intersect);

	      if(response){
		// An intersection was found
		distance =
		  calculateSqrdDistancePP(x, y, z,
					  intersect);
		if(distance < tmpdistance){
		  tmpdistance = distance;
		  // [OPTIMIZE]: Expensive
		  memcpy(active_intersection, intersect, sizeof(float)*3);
		  vertex_intersection_entity = e1;
		  found = true;
		}
	      }
	    }
	  }
	}
      }
    }
  }

  return 0;
}

// VERY naive function that attempts to find the closer entity to a click
// Travels through every point and stores the closest one
// [BUGBAIT]
int entityMarshall::findByCoordinates(float x, float y, float z){

  using idorumath::calculateSqrdDistancePP;
  using idorumath::calculateSqrdDistancePL;

  bool response = false;

  int ret_entity = 0;
  float distance = 0.f;
  // A bignum; distance^2 will probably never be bigger than this [FIXME]
  float tmpdistance = REALLYBIGFLOAT;

  for(unsigned int i = 0; i < entities_count; ++i){
    switch((cards_array + i)->gltype){

    case GL_LINES:

      for(unsigned int j = 0; j < (cards_array + i)->size; j += 2){

	response = 
	  calculateSqrdDistancePL(x, y, z, 
				  (cards_array + i)->data + j*3,
				  &distance);
	  
	if(!response){
	  distance = calculateSqrdDistancePP(x, y, z,
					     (cards_array + i)->data + j*3);
	  if(distance < tmpdistance){
	    tmpdistance = distance;
	    ret_entity = i;
	    setActiveVertex(i, (cards_array + i)->data + j*3);
	  }
	  distance = 
	    calculateSqrdDistancePP(x, y, z,
				    (cards_array + i)->data + j*3 + 3);
	  if(distance < tmpdistance){
	    tmpdistance = distance;
	    ret_entity = i;
	    setActiveVertex(i, (cards_array + i)->data + j*3 + 3);
	  }
	}
	
	if(response && distance < tmpdistance){
	  tmpdistance = distance;
	  ret_entity = i;
	  setActiveVertex(i, (cards_array + i)->data + j*3); // [FIXME]
	} 
      }
    break;

    case GL_LINE_STRIP:
      for(unsigned int j = 0; j < (cards_array + i)->size; ++j){
	
	// Let's assume first that the vertex are away from each other
	// and try to find the length of the perpendicular
	if(j < (cards_array + i)->size - 1){
	  response = 
	    calculateSqrdDistancePL(x, y, z, 
				    (cards_array + i)->data + j*3,
				    &distance);
	}

	// If the vertex are very close to each other 
	// the perpendicular will be more often out of the segment
	// and scanning for vertex will be more productive
	if(!response){
	  distance = 
	    calculateSqrdDistancePP(x, y, z,
				    (cards_array + i)->data + j*3);
	  if(distance < tmpdistance){
	    tmpdistance = distance;
	    ret_entity = i;
	    setActiveVertex(i, (cards_array + i)->data + j*3);
	  }
	}
	  
	if(response && distance < tmpdistance){
	  tmpdistance = distance;
	  ret_entity = i;
	  setActiveVertex(i, (cards_array + i)->data + j*3); // [FIXME]
	} 
      }
      break;

    default: 
      break;
    }
  }

  return ret_entity;

}
    

void entityMarshall::pushEntity(GLenum ltype,
				float* ldata,
				float* lnormals,
				float* ltexture,
				unsigned int layer){

  if (entities_count >= ENTITIES_IN_PAGE){
    err.quit("entityMarshall::pushEntity: Too many entities");
  }
  if (layer >= 256){
    err.quit("entityMarshall::pushEntity: Layer out of bounds");
  }
     
  // Be careful: all these should be already memory managed
  (cards_array + entities_count)->gltype = ltype;
  (cards_array + entities_count)->data = ldata;
  (cards_array + entities_count)->normals = lnormals;
  (cards_array + entities_count)->texture = ltexture;
  (cards_array + entities_count)->color = colors_array + (4 * layer);

}

void entityMarshall::updateEntity(unsigned int e,
				  GLenum ltype,
				  float* ldata,
				  float* lnormals,
				  float* ltexture,
				  unsigned int layer){

  if (e >= ENTITIES_IN_PAGE){
    err.quit("entityMarshall::pushEntity: Too many entities");
  }
  if (layer >= 256){
    err.quit("entityMarshall::pushEntity: Layer out of bounds");
  }
     
  // Be careful: all these should be already memory managed
  (cards_array + e)->gltype = ltype;
  (cards_array + e)->data = ldata;
  (cards_array + e)->normals = lnormals;
  (cards_array + e)->texture = ltexture;
  (cards_array + e)->color = colors_array + (4 * layer);
  
}

void entityMarshall::fixEntity(void){
  if((cards_array + entities_count)->data != 0){
    // Another use of the controversial entities_count.
    // This not-so-good approach is responsible of the ugly syntax (minus-1)
    // and doesn't handle the bug case "create triangles-create lines 
    // (overwrite: as if triangles weren't created)-fix lines-fix triangles"
    this->incrEntities();
  } else {
    // Kludge (avoid heap corruption)
    // This problem happens when new*Entity was called but no point
    // was pushed, and a (different) new*Entity is called later.
    // (nested nt-nl-l-t bug)
    err.quit("entityMarshall::fixEntity: Rogue null pointer");
  }
}

void entityMarshall::moveVertex(float x, float y, float z){

  // WARNING: TESTING
  // A little rant: this *should* be safe, since active_vertex 
  // should never be out of the memory pages space; however
  // that can't be throughly proven yet. And this destroys the 
  // previous position of the vertex: it will have to be cached.
  if(active_vertex != NULL){
    *active_vertex = x;
    *(active_vertex + 1) = y;
    *(active_vertex + 2) = z;
  }
  //END WARNING

}

// Returns true and sets snap_vertex
// if a snap was found for the given coordinates
// [TODO]: return what kind of snap was it
// Heavily unoptimized; when many primitives share a point
// it will be processed each time
// [TODO]: visual feedback when snapping
// Snaps done for END in GL_LINES, GL_LINE_STRIP
bool entityMarshall::findClosestSnap(float x, float y, float z){

  using idorumath::calculateSqrdDistancePP;

  bool found = false;

  float tmpdistance = 0.f;
  float sqrd_snap = 
    snap_distance * snap_distance * 
    scale_over_viewport * scale_over_viewport;
  float distance = sqrd_snap;

  for(unsigned int i = 0; i < entities_count; ++i){
    switch((cards_array + i)->gltype){

    case GL_LINES:

      // For each line...
      for(unsigned int j = 0; j < (cards_array + i)->size; j += 2){

	// First point...
	tmpdistance = 
	  calculateSqrdDistancePP(x, y, z, 
				  ((cards_array + i)->data) + (j * 3));

	if( tmpdistance < distance && tmpdistance < sqrd_snap ){
	  // Assign the found vertex data to snap_vertex
	  // They don't share address
	  // Possible optimization: store an index, don't memcopy until sure
	  memcpy(snap_vertex, 
		 ((cards_array + i)->data) + (j * 3), 
		 sizeof(float)*3);
	  distance = tmpdistance;
	  found = true;
	}

	// Second point...
	tmpdistance = 
	  calculateSqrdDistancePP(x, y, z, 
				  ((cards_array + i)->data) + ((j+1) * 3));

	if( tmpdistance < distance && tmpdistance < sqrd_snap ){
	  // Assign the found vertex data to snap_vertex
	  // They don't share address
	  memcpy(snap_vertex, 
		 ((cards_array + i)->data) + ((j+1) * 3), 
		 sizeof(float)*3);
	  distance = tmpdistance;
	  found = true;
	}

      }
    break;

    case GL_LINE_STRIP:
      for(unsigned int j = 0; j < (cards_array + i)->size; ++j){

	tmpdistance = 
	  calculateSqrdDistancePP(x, y, z, 
				  ((cards_array + i)->data) + (j * 3));

	if( tmpdistance < distance && tmpdistance < sqrd_snap ){
	  // Assign the found vertex data to snap_vertex
	  // They don't share address
	  memcpy(snap_vertex, 
		 ((cards_array + i)->data) + (j * 3), 
		 sizeof(float)*3);
	  distance = tmpdistance;
	  found = true;
	}
				
      }
      break;

    default: 
      break;
    }
  }

  return found;
}

// Strange idea - check that the cursor is inside both 
// boundboxes before testing for intersection. (works)
// Note: the bug of intersection not found on lineend is still
// active and very noticeable on test cases for this function

// Z unused right now
bool entityMarshall::pointinbox(float x, float y, float,
				unsigned int e1, unsigned int e2){

  if(((cards_array + e1) == 0) || ((cards_array + e2) == 0)){
    err.quit("entityMarshall::pointinbox: Bad entity page");
  }

  // May fail when the click was done from the wrong side of 
  // the nearest intersection.
  // May be solved adding to top, right, left, bottom
  // a distance equivalent to the snap ratio (fixed here)
  float margin = snap_distance * scale_over_viewport;

  if(((x <= ((cards_array + e1)->right + margin)) && 
      (x >= ((cards_array + e1)->left - margin)) &&
      (x <= ((cards_array + e2)->right + margin)) && 
      (x >= ((cards_array + e2)->left - margin))
      ) &&
     ((y <= ((cards_array + e1)->top + margin)) && 
      (y >= ((cards_array + e1)->bottom - margin)) &&
      (y <= ((cards_array + e2)->top + margin)) && 
      (y >= ((cards_array + e2)->bottom - margin))
      ))
    {
      return true;
    } else {
      return false;
    }
}

void entityMarshall::initBoundbox2d(float x,float y, float){

  if((cards_array + entities_count-1) == 0) 
    err.quit("entityMarshall::initBoundbox2d: Bad entity card");
  
  (cards_array + entities_count-1)->top = y;
  (cards_array + entities_count-1)->bottom = y;
  (cards_array + entities_count-1)->left = x;
  (cards_array + entities_count-1)->right = x;
}

void entityMarshall::recalculateBoundbox2d(float x, float y, float) {

  if((cards_array + entities_count-1) == 0) 
    err.quit("entityMarshall::recalculateBoundbox2d: Bad entity card");
  
  if (y > (cards_array + entities_count-1)->top){
    (cards_array + entities_count-1)->top = y;
  } // top

  if (y < (cards_array + entities_count-1)->bottom){
    (cards_array + entities_count-1)->bottom = y;
  } // bottom

  if (x < (cards_array + entities_count-1)->left){
    (cards_array + entities_count-1)->left = x;
  } // left 

  if (x > (cards_array + entities_count-1)->right){
    (cards_array + entities_count-1)->right = x;
  } // right

}

void entityMarshall::setSelected(unsigned int i){
  if(i < entities_count){
    selected_entity = i;
    // Shift the selected entity queue, which is a [xyzn] uint*
    // where n is displaced ([xyzn] --> [ixyz])
    memmove(selected_entity_array+1, 
	    selected_entity_array, 
	    3*sizeof(unsigned int));
    selected_entity_array[0] = i;
  }
}

float const * const entityMarshall::getEntity(int i, 
					      GLenum & type, int & size){
  // [FIXME]: will need non-gl types to represent circles, arcs, etc.
  type = (cards_array + i)->gltype;
  size = (cards_array + i)->size;
  return (cards_array + i)->data;
}

// Header codelets

void entityMarshall::incrPoints(int n) { points_count += n; }
void entityMarshall::incrEntities(void) { ++entities_count; }
void entityMarshall::incrEntities(int n) { entities_count += n; }
void entityMarshall::incrFaces(int n) { faces_count += n; }
void entityMarshall::incrLines(int n) { lines_count += n; }
void entityMarshall::incrSelected(void){ ++(selected_entity_array[0]); }
void entityMarshall::decrSelected(void){ --(selected_entity_array[0]); }
void entityMarshall::setViewportScale(float vs){ scale_over_viewport = vs; }
void entityMarshall::setSelect(bool b){ select = b; }
void entityMarshall::setSnap(bool b){ snap = b; }
void entityMarshall::setSnapRatio(float f){ snap_distance = f; }
void entityMarshall::setLighting(bool b){ lighting = b; }
void entityMarshall::setBlending(bool b){ blending = b; }
void entityMarshall::setTexture(bool b){ texture = b; }

void entityMarshall::reset(void) { 
  points_count = 0;
  entities_count = 0;
  faces_count = 0;
  lines_count = 0;
}

int entityMarshall::getNumPoints(void) { return points_count; }
int entityMarshall::getNumEntities(void) { return entities_count; }
int entityMarshall::getSize(void) { return entities_count; }
int entityMarshall::getNumFaces(void) { return faces_count; }
int entityMarshall::getNumLines(void) { return lines_count; }
int entityMarshall::getSelected(void){ return selected_entity_array[0]; }
bool entityMarshall::snapState(void){ return snap; }

float const * const entityMarshall::getSnapVertex(void){ 
  return snap_vertex; }
float const * const entityMarshall::getIntVertex(void){ 
  return active_intersection; }

float const * const entityMarshall::getLastIntersection(void){
  return active_intersection; 
}

// Very serious problem; it increases the latest entity always, which is
// not the desired case since out there are four active entities at any
// given moment. If an entity different of the latest one has a point 
// pushed, the latest one will attempt to draw beyond its initialized segment
void entityMarshall::incrSize(void) { 
  if((cards_array + (entities_count - 1)) == 0) 
    err.quit("entityMarshall::incrSize: Bad entity card");

  // Substitute here (entities_count - 1) for the appropiate one, according
  // to the argument (which will not be 'void')
  ++((cards_array + (entities_count - 1))->size); 
}

void entityMarshall::incrSize(unsigned int n) { 
  if (n >= entities_count)
    err.quit("entityMarshall::incrSize: Bad entity card");

  ++((cards_array + n)->size); 
}

void entityMarshall::setColorsArray(float* colors){ 
  if (colors == 0) err.quit("Bad colors array");
  colors_array = colors;
}

void entityMarshall::applyBlending(bool b){
  if (b){
    if (blending) glEnable(GL_BLEND);
  } else
    if (!blending) glDisable(GL_BLEND);
}

void entityMarshall::applyTexture(bool b){
  if (b){
    if (texture) glEnable(GL_TEXTURE_2D);
  } else
    if (!texture) glDisable(GL_TEXTURE_2D);
}

void entityMarshall::applyLighting(bool b){
  if (b){
    if (lighting) glEnable(GL_LIGHTING);
  } else
    if (!lighting) glDisable(GL_LIGHTING);
}

// END Header codelets
