/* 
 *    python-cad, a GL rendered, python scripted drawing engine
 *                  Copyright (C) 2002 by
 *
 *      Jaime Soffer             (jsoffer@soffernet.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef IMATH_H
#define IMATH_H 1 

#include <math.h>

namespace idorumath {

  /**
   * If an intersection exists returns true and sets 
   * float* intersection to it; else returns false and 
   * doesn't touch it
   */
  bool findLinesIntersection(float const * const xyz1,
			     float const * const xyz2,
			     float* intersection);

  /**
   * Returns the distance between two [xyz] points
   */
  float calculateDistancePP(float* fp1, float* fp2);

  /**
   * Returns the squared distance between two [xyz] points.
   * Faster for comparison.
   */
  float calculateSqrdDistancePP(float* fp1, float* fp2);

  /**
   * Returns the distance between a x, y, z point and a [xyz] float*
   */
  float calculateDistancePP(float x, float y, float z, 
			    float * fp);

  /**
   * Returns the squared distance between a x, y, z point and a [xyz] float*
   * Faster for comparison
   */
  float calculateSqrdDistancePP(float x, float y, float z, 
				float * fp);

  /**
   * Square of distance between point [xyz] and double vector6 [[xyz][xyz]]
   * Loosely based on 
   * http://astronomy.swin.edu.au/~pbourke/geometry/pointline/source.c
   * Returns 0 if the perpendicular doesn't intersect, 
   * cueing for more calculation
   */
  int calculateSqrdDistancePL(float x, float y, float z,
				float * fp, 
				float * ret);


  class Quaternion {
  private:
    float w, x, y, z;

  public:
    Quaternion(void);
    Quaternion(float, float, float, float z);
    ~Quaternion(void);

    void setData(float, float, float, float);
    /*
    void toGLMatrix(float*);
    */

    Quaternion operator*(Quaternion q);
  };

}

#endif
