/* 
 *    python-cad, a GL rendered, python scripted drawing engine
 *                  Copyright (C) 2002, 2003 by
 *
 *      Jaime Soffer             (jsoffer@soffernet.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef IMEM_H
#define IMEM_H 1

class iMem;

#include "entitymarshall.hpp"

class iMem {
  
 private:

  GLenum array_type; // GL_POINTS, GL_LINES, GL_LINE_STRIP, etc.

  iError err;

  entityMarshall* marshall;

  unsigned int index; // Holds the current point available to be filled
  unsigned int entity_index; // The position on the current entity

  unsigned int active_entity;

  // To save vertex values here
  float* data; 
  //  float* colors;
  float* normals;
  float* texture;

  // If an entity is broken between pages, remember its layer
  unsigned int last_layer;
  unsigned int* entity_size; // What's the current size of the entity?

  // avoid making empty entities
  bool double_beginentity_test;

  bool initialized;

  bool duplicated;
  bool first_point_in_entity;

  // control the winding on the triangle strips
  bool winding;

  // For duplicating points on broken triangle strips
  // Line strips need only point duplication through next->pushPoint
  float pointprev_x, pointprev_y, pointprev_z;


  // Check if the last entity continues on the next page
  //  bool incomplete;

  //  bool* isPointAlive; // Array to flag points that are to be removed

  // Implement these to make faster to push a point on an advanced page
  //  iMem* head; 
  //  iMem* tail;

  iMem* prev;
  iMem* next; // To use in a (doubly) linked list
  //  iMem* ViewportNext; // The original linked list will be kept untouched 
                      // while only the visible pages are passed to
                      // iCore->Draw(); this pointer is to make a second
                      // list that holds only the drawable pages
                      // (Port to EntityMarshall?)

  /* Used to determine if the current page is important to functions */
  /* If min_x is higher than the right of the screen, for example,   */
  /* ignore the page for further processing */ 
  //  float min_x, min_y, max_x, max_y;

  // Inline function, takes the address of an array containing 
  // nine floats defining a 3d triangle, calculates the cross product
  // and stores the normal vector as three floats on another array.
  // Dangerous if not used correctly. Handle with appropiate safety 
  // equipment. Slippery when wet.
  // ******** normals_from_triangles(to, from) 3vector <- 3x3rowmatrix *****
  // Was altered to be closer in behavior to memcpy(dest, src, n);
  // maybe would be good to implement the 'n' part to handle many at once
  // [BUGBAIT]
  void normals_from_triangles(float* normals, const float* const triangles){
    *(normals + 0) = // x = (y1*z2)-(z1*y2)
      (( *(triangles + 4) - *(triangles + 1) ) *  // y1
       ( *(triangles + 8) - *(triangles + 2) )) -  // z2
      (( *(triangles + 5) - *(triangles + 2) ) *  // z1
       ( *(triangles + 7) - *(triangles + 1) ));   // y2
    *(normals + 1) = // y = (z1*x2)-(x1*z2)
      (( *(triangles + 5) - *(triangles + 2) ) *  // z1
       ( *(triangles + 6) - *(triangles + 0) )) -  // x2
      (( *(triangles + 3) - *(triangles + 0) ) *  // x1
       ( *(triangles + 8) - *(triangles + 2) ));   // z2
    *(normals + 2) = // z = (x1*y2)-(y1*x2)
      (( *(triangles + 3) - *(triangles + 0) ) *  // x1
       ( *(triangles + 7) - *(triangles + 1) )) -  // y2
      (( *(triangles + 4) - *(triangles + 1) ) *  // y1
       ( *(triangles + 6) - *(triangles + 0) ));  // x2
    
    // Extra paranoia: if the normal vector is null, stop here
    if( *(normals + 0) || *(normals + 1) || *(normals + 2) ) {
      float normal_factor = 
	sqrt(((*(normals + 0)) * (*(normals + 0))) + 
	     ((*(normals + 1)) * (*(normals + 1))) +
	     ((*(normals + 2)) * (*(normals + 2))));

      if(normal_factor){       // won't divide 0/0
	*(normals + 0) /= normal_factor;
	*(normals + 1) /= normal_factor;
	*(normals + 2) /= normal_factor;
      }
    }
  }                          

 public:

  /* Construction */
  // This is the head of a linked list
  iMem(GLenum page_type, entityMarshall* leader); 
  iMem(iMem* previous); // and this follows it and grabs info from it
  ~iMem(void);

  /* Execution */
  void pushPoint(float x, float y, float z);
  void beginEntity(unsigned int layer);

  /* Execution -- TO_DO */
  /*
  void popPoint(void);
  insert_point(int position);
  void delete_point(int position);
  */

  /* Accessors */

  // Pointers should return as constant... later

  iMem* getNext(void) { return next; };
  entityMarshall* getMarshall(void) { return marshall; };
  GLenum const getArrayType(void) { return array_type; };
  unsigned int const getLastLayer(void){return last_layer;};
  //  unsigned int const getIndex(void) { return index; };
  //  unsigned int const getEntityIndex(void){return entity_index;};
  float const *getData(void) { return data; };
  float const *getNormals(void) { return normals; };
  float const *getTexture(void) { return texture; };

  // Estas funciones permiten acceso directo de solo lectura
  // a la memoria que define a cada entidad. (Unused?)
  /*
  void setCurrentData(void);
  void getCurrentNormals(void);
  void getCurrentTex(void);
  */

  unsigned int const *getEntitySize(void){ return entity_size; };
  // unsigned int const *getEntityLayer(void){ return entity_layer; };

  /* Accessors -- TO_DO */

  //  float const *getColors(void) { return colors; };
  //  iMem* getHead(void) { return head; };
  //  iMem* getTail(void) { return tail; };
  //  void setTail(iMem* updated_tail) { tail = updated_tail; };
  //  void importPreviousLastVertex(float* ret);


};

#endif
