/* 
 *    fandango, a GL rendered, python scripted drawing engine
 *                  Copyright (C) 2002, 2003 by
 *
 *      Jaime Soffer             (jsoffer@soffernet.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "itexture.hpp"
#include <qgl.h>

/* For a while I'm only addind one kind of texture */

iTexture::iTexture(void){
  data = new unsigned char [256*256*3]; err.check(data);
  if(data == NULL) err.quit("iTexture: badly assignated ponter");
  memset(data, 0, sizeof(unsigned char)*256*256*3);
}

iTexture::~iTexture(void){
  if(data != NULL){
    delete [] data; data = NULL;
  }
}

void iTexture::init(void){
  glBindTexture(GL_TEXTURE_2D, 13);
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  glTexImage2D (GL_TEXTURE_2D, 
		0, 
		3, 
		256, 256,
		0, 
		GL_RGB, GL_UNSIGNED_BYTE, 
		data);
}

/* This, in python, makes a 4x4 checkerboard (40 char width):

for i in range(1000):
...     if (i/(40*4))%2:
...             print (i/4)%2,
...     else:
...             print not (i/4)%2,

*/

void iTexture::makeCheckerboard(void){
  for(int i = 0; i < (256*256); ++i){

    if((i/(32*256))%2){
      *(data + i*3 + 0) = 255 * ((i/32)%2);
      *(data + i*3 + 1) = 255 * ((i/32)%2);
      *(data + i*3 + 2) = 255 * ((i/32)%2);
    } else {
      *(data + i*3 + 0) = 255 * !((i/32)%2);
      *(data + i*3 + 1) = 255 * !((i/32)%2);
      *(data + i*3 + 2) = 255 * !((i/32)%2);
    }
  }
}
