/* 
 *    fandango, a GL rendered, python scripted drawing engine
 *                  Copyright (C) 2002, 2003 by
 *
 *      Jaime Soffer             (jsoffer@soffernet.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "kpyaction.hpp"


KPyAction::KPyAction(QPyObject* qpo, QString command,
	  const QString& text, const QString& pix, 
	  const KShortcut& cut, 
	  const QObject* receiver, const char* slot, 
	  KActionCollection* parent, const char* name ) :
  KAction(text, pix, cut, receiver, slot, parent, name) {

  python_command = command;

  connect(this, SIGNAL( activated() ),
	  this, SLOT(called()));
  connect(this, SIGNAL( executeCommand(const QString &) ),
	  qpo, SLOT(processExternalCommand(const QString &)));

}

// Quick hack (the icons load correctly now)
KPyToggleAction::KPyToggleAction(QPyObject* qpo, 
				 QString command_on, QString command_off,
				 const QString& text, 
				 const QString& pix, // The hack
				 const KShortcut& cut, 
				 const QString& group = 0,
				 QObject* parent = 0, 
				 const char* name = 0)
  // 0, 0 means connect this action to the null slot of a null widget 
  // (not used)
  : KToggleAction(text, pix, cut, 0, 0, parent, name){

  if(group){
    setExclusiveGroup(group);
  }

  python_command_on = command_on;
  python_command_off = command_off;

  connect(this, SIGNAL( toggled(bool) ),
	  this, SLOT(called(bool)));
  connect(this, SIGNAL( executeCommand(const QString &) ),
	  qpo, SLOT(processExternalCommand(const QString &)));

}

void KPyAction::called(){
  emit executeCommand(python_command);
}

void KPyToggleAction::called(bool b){
  if(b){
    emit executeCommand(python_command_on);
  } else {
    emit executeCommand(python_command_off);
  }
}

#include "kpyaction.moc"
