/* 
 *    fandango, a GL rendered, python scripted drawing engine
 *                  Copyright (C) 2002, 2003 by
 *
 *      Jaime Soffer             (jsoffer@soffernet.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef KPYACTION_H
#define KPYACTION_H

#include "qpyobject.hpp"
#include <kaction.h>

/**
 * Special action that is set up to run Python commands.
 */
class KPyAction : public KAction
{
  Q_OBJECT

private:
  QString python_command;

public:

  /**
   * The main difference between this and the base KAction constructor
   * are the QPyObject*, that points to a running python interpreter,
   * and next to it a QString, the python command to be executed 
   * when the function is activated.
   */
  /*
  KPyAction(QPyObject*, QString,
	    const QString&, const QIconSet&, const KShortcut&,
	    QObject*, const char*);
  */

  KPyAction(QPyObject*, QString,
	    const QString& text, const QString& pix, 
	    const KShortcut& cut, 
	    const QObject* receiver, const char* slot, 
	    KActionCollection* parent, const char* name );

  public slots:
  /**
   * A bit of a kludge. It's a bridge between the 'activated()' and the
   *  'executeCommand(QString)' signals.
   */
  void called(void);

  signals:

  /**
   * Hook it to a QPyObject and it allows executing python commands
   * between classes.
   */
  void executeCommand(const QString &);
  
};

/**
 * Special action that is set up to run Python commands.
 */
class KPyToggleAction : public KToggleAction
{
  Q_OBJECT

private:
  QString python_command_on;
  QString python_command_off;

public:

  /**
   * The main difference between this and the base KAction constructor
   * are the QPyObject*, that points to a running python interpreter,
   * and next to it two QStrings, python commands to be executed 
   * when the function is toggled on or off.
   */

  /*
  KPyToggleAction(QPyObject*, QString, QString,
		  const QString&, const QIconSet&, const KShortcut&,
		  QObject*, const char*);

  KPyToggleAction(QPyObject*, QString, QString,
		  const QString&, const QIconSet&, const KShortcut&,
		  const QString&, QObject*, const char*);
  */

  // Qick hack to change the iconset to a name
  KPyToggleAction(QPyObject*, QString, QString,
		  const QString&, const QString&, const KShortcut&,
		  const QString&, QObject*, const char*);

  public slots:
  /**
   * A bit of a kludge. It's a bridge between the 'activated()' and the
   *  'executeCommand(QString)' signals.
   */
  void called(bool);

  signals:

  /**
   * Hook it to a QPyObject and it allows executing python commands
   * between classes.
   */
  void executeCommand(const QString &);
  
};

#endif
