/* 
 *    fandango, a GL rendered, python scripted drawing engine
 *                  Copyright (C) 2002, 2003 by
 *
 *      Jaime Soffer             (jsoffer@soffernet.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//#include <qnamespace.h>

#include "kfandango.hpp"
#include "kfandangowidget.hpp"

#include <kcmdlineargs.h>
#include <kaboutdata.h>

#include <klocale.h>
#include <kapp.h>
#include <kmainwindow.h>

#include <kmessagebox.h>

// Global pointer, to make it accessible to the extension 
// (can't be made static)
// It is now also 'extern'ed to the KFandango class [FIXME]
KFandangoWidget * gw;

static KCmdLineOptions options[] =
{
  { "nodotfile", I18N_NOOP("Ignores the dotfile"), 0 },
  { 0, 0, 0 }
  // INSERT YOUR COMMANDLINE OPTIONS HERE
};

int main( int argc, char **argv )
{

  // Make more flexible the version input
  KAboutData aboutData("kfandango", I18N_NOOP("Fandango"), "0.2.5",
		       I18N_NOOP("Python scripted 3d CAD"), 
		       KAboutData::License_GPL_V2,
		       "(c) 2001-2003, Jaime Soffer", 
		       /*I18N_NOOP("freeform text"),*/ 0,
		       "http://www.soffernet.com/jaime/fandango",
		       "jsoffer@soffernet.com");
  aboutData.addAuthor("Jaime Soffer", 
		      I18N_NOOP("Main developer"),
		      "jsoffer@soffernet.com", 
		      "http://www.soffernet.com/jaime");
  KCmdLineArgs::init( argc, argv, &aboutData );
  KCmdLineArgs::addCmdLineOptions( options );

  KApplication::setColorSpec(KApplication::CustomColor);
  KApplication a;

  if ( !QGLFormat::hasOpenGL() ) {
    KMessageBox::error(0, "This system has no OpenGL support. Exiting.",
		       "Fandango: GL error");
    exit(1);
  }

  KFandango* mainapp = new KFandango(argc, argv,
				     0L, "Fandango");

  a.setMainWidget(mainapp);

  mainapp->show();

  // Here the order of initialization is important
  // This calls the base python modules and the dotfile
  // (If the agrument is false the dotfile is not called)
  KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
  mainapp->initialize(args->isSet("dotfile"));

  kdDebug() << "Before a.exec()" << endl;
  return a.exec();
  kdDebug() << "After a.exec()" << endl;
}
