/* 
 *    fandango, a GL rendered, python scripted drawing engine
 *                  Copyright (C) 2002, 2003 by
 *
 *      Jaime Soffer             (jsoffer@soffernet.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// Python extensions

// Notes: on the tutorials the functions are defined as static.



#include "primitives.hpp"
#include "kfandangowidget.hpp"

extern KFandangoWidget* gw;

extern "C" {

static PyMethodDef PycadInputMethods[] = {
  {
    "mousebutton",
    null_placeholder_function,
    METH_VARARGS,
    "Holder function. Will be called each time a mouse button is pressed"
  },
  {
    "mouseLbutton",
    null_placeholder_function,
    METH_VARARGS,
    "Holder function. Will be called when the Left Mouse Button is pressed"
  },
  {
    "mouseMbutton",
    null_placeholder_function,
    METH_VARARGS,
    "Holder function. Will be called when the Middle Mouse Button is pressed"
  },
  {
    "mouseRbutton",
    null_placeholder_function,
    METH_VARARGS,
    "Holder function. Will be called when the Right Mouse Button is pressed"
  },
  {
    "clearcalled",
    null_placeholder_function,
    METH_VARARGS,
    "Holder function. Will be called each time iCore::clear is called"
  },
  { NULL, NULL, 0, NULL }
};

PyMethodDef cadgetMethods[] = {
  {
    "ratio",
    ratio,
    METH_VARARGS,
    "Returns the current window ratio"
  },
  {
    "clicks",
    clicks,
    METH_VARARGS,
    "Returns a tuple with the three last clicked points"
  },
  { /* Not implemented? */
    "snaps",
    snaps,
    METH_VARARGS,
    "Returns a tuple with already snapped clicks [PENDING]"
  },
  {
    "select_entity",
    find_entity,
    METH_VARARGS,
    "Finds the closer entity to a point"
  },
  {
    "set_selected",
    set_selected,
    METH_VARARGS,
    "Sets the selected entity"
  },
  {
    "get_selected",
    get_selected,
    METH_VARARGS,
    "Retrieves the index of the selected entity"
  },
  {
    "get_int",
    get_intersection,
    METH_VARARGS,
    "Retrieves a tuple [xyz] of the last found intersection"
  },
  {
    "get_entity",
    get_entity,
    METH_VARARGS,
    "When done, should return a tuple with the data of an entity"
  },
  {
    "get_errors",
    get_errors,
    METH_VARARGS,
    "Detailed inform of the latest error messages"
  },
  {
    "data_path",
    datapath,
    METH_VARARGS,
    "Returns the path to the shared files (fonts and scripts)"
  },
  { NULL, NULL, 0, NULL }
};

PyMethodDef GLStateMethods[] = {
  {
    "axis",
    rotation_axis,
    METH_VARARGS,
    "Defines the axis vector for rotations"
  },
  {
    "reset",
    reset,
    METH_VARARGS,
    "Resets axis transformations"
  },
  {
    "lock",
    lock,
    METH_VARARGS,
    "When true, the screen can't be rotated with the mouse"
  },
  {
    "rot_style",
    rot_style,
    METH_VARARGS,
    "Chooses between free and axial rotation"
  },
  {
    "zoom",
    viewport_zoom,
    METH_VARARGS,
    "Sets the left, top, bottom viewport borders"
  },
  {
    "clearcolor",
    bg,
    METH_VARARGS,
    "Sets the RGB clear color"
  },
  {
    "color",
    set_color,
    METH_VARARGS,
    "Sets the RGB foreground color"
  },
  {
    "lcolor",
    set_light_color,
    METH_VARARGS,
    "Sets the RGB color for a named light"
  },
  {
    "lpos",
    set_light_position,
    METH_VARARGS,
    "Sets the position for a named light"
  },
  {
    "linew",
    set_line_width,
    METH_VARARGS,
    "Sets the line width"
  },
  {
    "aa",
    line_antialiasing,
    METH_VARARGS,
    "Turns dithering on/off"
  },
  {
    "blend",
    blending,
    METH_VARARGS,
    "Turns blending on/off"
  },
  {
    "texture",
    texture,
    METH_VARARGS,
    "Turns texturing on/off"
  },
  {
    "light",
    lighting,
    METH_VARARGS,
    "Turns lighting on/off"
  },
  {
    "select",
    highlight,
    METH_VARARGS,
    "Switches select mode"
  },
  {
    "cull",
    cull,
    METH_VARARGS,
    "Culling switch"
  },
  {
    "getsnap",
    getsnap,
    METH_VARARGS,
    "Switches snap mode"
  },
  {
    "getsnapvertex",
    getsnapvertex,
    METH_VARARGS,
    "Returns the last snap found"
  },
  { /* Unused? */
    "snap",
    setsnap,
    METH_VARARGS,
    "Switches snap mode"
  },
  { /* What does it do here? [FIXME] */
    "findint",
    findint,
    METH_VARARGS,
    "Finds the closest intersection to [xyz]"
  },
  { /* It's a duplicate of get_int*/
    "getint",
    getint,
    METH_VARARGS,
    "Returns the last found intersection"
  },
  {
    "snap_ratio",
    setsnapratio,
    METH_VARARGS,
    "Sets the ratio of the snap area"
  },
  { /* Again, what is it for? ("snap") */
    "snapsetp",
    getsnapstate,
    METH_VARARGS,
    "Returns current state of the snap flag"
  },
  { /* Disabled? */
    "update",
    update,
    METH_VARARGS,
    "Makes a forced update of the state of the screen"
  },
  { NULL, NULL, 0, NULL }
};

PyMethodDef FandangoOutMethods[] = {
  {
    "write",
    stdout_write,
    METH_VARARGS,
    "Substitutes sys.stdout.write(s)"
  },
  { NULL, NULL, 0, NULL }
};

PyMethodDef FandangoErrMethods[] = {
  {
    "write",
    stderr_write,
    METH_VARARGS,
    "Substitutes sys.stderr.write(s)"
  },
  { NULL, NULL, 0, NULL }
};

PyMethodDef DrawingMethods[] = {
    {
      "clear",
      clear,
      METH_VARARGS,
      "Deletes all lines"
    },
    {
      "line",
      line,
      METH_VARARGS,
      "Draws a line from two points"
    },
    {
      "triangle",
      triangle,
      METH_VARARGS,
      "Draws a triangle between three points"
    },
    {
      "line_from",
      (PyCFunction)line_from, // Needs a cast
      METH_VARARGS | METH_KEYWORDS,
      "Begins a new line strip"
    },
    {
      "line_to",
      line_to,
      METH_VARARGS,
      "Continues a line strip"
    },
    {
      "triangle_from",
      tris_from,
      METH_VARARGS,
      "Begins a new triangle strip"
    },
    {
      "triangle_to",
      tris_to,
      METH_VARARGS,
      "Continues a triangls strip"
    },
    {
      "beginTrianglesEntity",
      beginTrianglesEntity,
      METH_VARARGS,
      "Marks the difference between entities and sets layers"
    },
    {
      "beginLinesEntity",
      beginLinesEntity,
      METH_VARARGS,
      "Marks the difference between entities and sets layers"
    },
    { NULL, NULL, 0, NULL }
};

static PyMethodDef EditMethods[] = {
  {
    "move",
    move,
    METH_VARARGS,
    "If a vertex is selected, move it to [xyz]"
  },
  { NULL, NULL, 0, NULL }
};

void initEdit(void){
(void) Py_InitModule("fandangoedit", EditMethods);
}

void initStdout(void){
(void) Py_InitModule("fandango_stdout", FandangoOutMethods);
}

void initStderr(void){
(void) Py_InitModule("fandango_stderr", FandangoErrMethods);
}

void initGLState(void){
(void) Py_InitModule("glstate", GLStateMethods);
}

void initDrawing(void){
(void) Py_InitModule("idorudraw", DrawingMethods);
}

void initGet(void){
(void) Py_InitModule("cadget", cadgetMethods);
}

void initInput(void){
(void) Py_InitModule("fandango_input", PycadInputMethods);
}

PyObject* stdout_write(PyObject *self, PyObject *args){

  char* s;
  if (!PyArg_ParseTuple(args, "s", &s)){
    return NULL;
  }

  gw->fakePythonStdout(s);

  Py_INCREF(Py_None); return Py_None;
}

PyObject* stderr_write(PyObject *self, PyObject *args){

  char* s;
  if (!PyArg_ParseTuple(args, "s", &s)){
    return NULL;
  }

  gw->fakePythonStderr(s);

  Py_INCREF(Py_None); return Py_None;
}

PyObject* bg(PyObject *self, PyObject *args){

  float r, g, b;
  if (!PyArg_ParseTuple(args, "fff", &r, &g, &b)){
    return NULL;
  }

  glClearColor(r, g, b, 1.0);

  Py_INCREF(Py_None); return Py_None;
}

// equivalente a la funcin GL
// optimizar para grupos de lneas

// stack drawing methods

PyObject* line_from(PyObject *self, PyObject *args, PyObject *keywds){
  float lx, ly;
  float lz = 0.f;
  int layer = 1;
  
  char* kwlist[] = { "x", "y", "z", "l", NULL };
  
  if (!PyArg_ParseTupleAndKeywords(args, keywds,
				   "ff|fi", kwlist,
				   &lx, &ly, &lz, &layer)){
    return NULL;
  }
  

  // [FIXME]: more NUM_LAYERS woes; let's define them at main.h :)
  // '192' must not be hardcoded; depends on a NUM_LAYERS value in iCore.cc
  if((layer <= 0) || (layer > 192)) layer = 1;
  gw->newStrip(lx, ly, lz, layer);
  
  Py_INCREF(Py_None); return(Py_None);
}
  
PyObject* line_to(PyObject *self, PyObject *args){
  float lx, ly;
  float lz = 0.f;
  if (!PyArg_ParseTuple(args, "ff|f", &lx, &ly, &lz)){
    return NULL;
  }
  gw->pushToStrip(lx, ly, lz);
  
  Py_INCREF(Py_None); return(Py_None);
}

PyObject* tris_from(PyObject *self, PyObject *args){
  float x1, y1, x2, y2;
  float z1 = 0.f;
  float z2 = 0.f;
  int layer = 1;
  
  if (!PyArg_ParseTuple(args, "ffff|ffi", 
			&x1, &y1, &x2, &y2, &z1, &z2, &layer)){
    return NULL;
  }
  
  // '254' must not be hardcoded; depends on a NUM_LAYERS value in iCore.cc
  if((layer <= 0) || (layer > 254)) layer = 1;
  gw->newTriStrip(x1, y1, z1, 
		       x2, y2, z2,
		       layer);
  
  Py_INCREF(Py_None); return(Py_None);
}

PyObject* tris_to(PyObject *self, PyObject *args){
  float lx, ly;
  float lz = 0.f;
  if (!PyArg_ParseTuple(args, "ff|f", &lx, &ly, &lz)){
    return NULL;
  }
  gw->pushToTriStrip(lx, ly, lz);
  
  Py_INCREF(Py_None); return(Py_None);
}
  
PyObject* beginTrianglesEntity(PyObject *self, PyObject *args){
  unsigned int l = 1;
  if (!PyArg_ParseTuple(args, "|i", &l)){
    return NULL;
  }
  
  gw->newTrianglesEntity(l);
  
  Py_INCREF(Py_None); return(Py_None);
}

PyObject* beginLinesEntity(PyObject *self, PyObject *args){
  unsigned int l = 1;
  if (!PyArg_ParseTuple(args, "|i", &l)){
    return NULL;
  }
  
  gw->newLinesEntity(l);
  
  Py_INCREF(Py_None); return(Py_None);
}


PyObject* line(PyObject *self, PyObject *args){

  kdDebug() << "line" << endl;
  float lx1, ly1, lx2, ly2;
  float lz1 = 0.f;
  float lz2 = 0.f;
  kdDebug() << "line: after variables, before parsetuple" << endl;
  if (!PyArg_ParseTuple(args, "ffff|ff", &lx1, &ly1, &lx2, &ly2, &lz1, &lz2)){
    return NULL;
  }
  kdDebug() << "line: after parsetuple, before pushline" << endl;
  gw->pushLine(lx1, ly1, lz1, lx2, ly2, lz2);
  kdDebug() << "line: after pushline, before return" << endl;
  Py_INCREF(Py_None); return(Py_None);
  kdDebug() << "line: after return" << endl;
}


PyObject* triangle(PyObject *self, PyObject *args){
  float x1, y1, x2, y2, x3, y3;
  float z1 = 0.f;
  float z2 = 0.f;
  float z3 = 0.f;
  if (!PyArg_ParseTuple(args, "ffffff|fff", &x1, &y1, &x2, &y2, &x3, &y3, &z1, &z2, &z3)){
    return NULL;
  }
  gw->pushTriangle(x1, y1, z1, x2, y2, z2, x3, y3, z3);
  
  Py_INCREF(Py_None); return(Py_None);
}

////////////////////////////// GL state wrappers ////////////////////////////

PyObject* set_line_width(PyObject *self, PyObject *args){
  int i;
  if (!PyArg_ParseTuple(args, "i", &i)){
    return NULL;
  }
  glLineWidth(i);
  
  Py_INCREF(Py_None); return(Py_None);
}

PyObject* set_color(PyObject *self, PyObject *args){
  float r, g, b;
  float a = 1.f;
  int layer = 1;
  if (!PyArg_ParseTuple(args, "fff|if", &r, &g, &b, &layer, &a)){
    return NULL;
  }

  gw->setLayerColor(layer, r, g, b, a);
  
  Py_INCREF(Py_None); return(Py_None);
}

PyObject* set_light_color(PyObject *self, PyObject *args){
  float r, g, b;
  float a = 1.f;
  int light = 0;
  if (!PyArg_ParseTuple(args, "fff|if", &r, &g, &b, &light, &a)){
    return NULL;
  }

  gw->setLightColor(light, r, g, b, a);
  
  Py_INCREF(Py_None); return(Py_None);
}

PyObject* set_light_position(PyObject *self, PyObject *args){
  float x, y, z;
  int light = 0;
  if (!PyArg_ParseTuple(args, "fff|i", &x, &y, &z, &light)){
    return NULL;
  }

  gw->setLightPosition(light, x, y, z);
  
  Py_INCREF(Py_None); return(Py_None);
}

PyObject* line_antialiasing(PyObject *self, PyObject *args) {
  int i; // bool
  if (!PyArg_ParseTuple(args, "i", &i)){
    return NULL;
  }
  
    if(i){
    glEnable(GL_BLEND);
    glEnable(GL_LINE_SMOOTH);
    } else {
    gw->applyBlending(false);
    glDisable(GL_LINE_SMOOTH);
    }

  Py_INCREF(Py_None); return Py_None;
}

PyObject* blending(PyObject *self, PyObject *args) {
  int i; // bool
  if (!PyArg_ParseTuple(args, "i", &i)){
    return NULL;
  }
  gw->setBlending(i);
  
  Py_INCREF(Py_None); return Py_None;
}

PyObject* texture(PyObject *self, PyObject *args) {
  int i; // bool
  if (!PyArg_ParseTuple(args, "i", &i)){
    return NULL;
  }
  gw->setTexture(i);
  
  Py_INCREF(Py_None); return Py_None;
}

PyObject* lighting(PyObject *self, PyObject *args) {
  int i;
  if (!PyArg_ParseTuple(args, "i", &i)){
    return NULL;
  }

  gw->setLighting(i);
    
  Py_INCREF(Py_None); return Py_None;
}

PyObject* highlight(PyObject *self, PyObject *args) {
  int i;
  if (!PyArg_ParseTuple(args, "i", &i)){
    return NULL;
  }

  gw->setSelect(i);
  
  Py_INCREF(Py_None); return Py_None;
}

PyObject* cull(PyObject *self, PyObject *args) {

  int i;
  if (!PyArg_ParseTuple(args, "i", &i)){
    return NULL;
  }

  if(i){
    glEnable(GL_CULL_FACE);
  } else {
    glDisable(GL_CULL_FACE);
  }

  Py_INCREF(Py_None); return Py_None;
}

/////////////////////////////// Accessors ///////////////////////////////////

PyObject* getsnap(PyObject *self, PyObject *args) {

  float x; float y;
  float z = 0.f;

  if (!PyArg_ParseTuple(args, "ff|f", &x, &y, &z)){
    return NULL;
  }

 return Py_BuildValue("i", gw->findSnap(x, y, z));
}

PyObject* findint(PyObject *self, PyObject *args) {

  float x; float y;
  float z = 0.f;

if (!PyArg_ParseTuple(args, "ff|f", &x, &y, &z)){
    return NULL;
  }

 gw->findIntersection(x, y, z);

 Py_INCREF(Py_None); return Py_None;
}

PyObject* getsnapvertex(PyObject *self, PyObject *args) {

if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }

 float const * const v = gw->getSnapVertex();

 return Py_BuildValue("[fff]", v[0], v[1], v[2]);

}

PyObject* getint(PyObject *self, PyObject *args) {

if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }

 float const * const v = gw->getIntVertex();

 return Py_BuildValue("[fff]", v[0], v[1], v[2]);

}

PyObject* setsnap(PyObject *self, PyObject *args) {

  int i;

  if (!PyArg_ParseTuple(args, "i", &i)){
    return NULL;
  }

  gw->setSnap(i);

  Py_INCREF(Py_None); return Py_None;
}

PyObject* setsnapratio(PyObject *self, PyObject *args) {

  float f;

  if (!PyArg_ParseTuple(args, "f", &f)){
    return NULL;
  }

  gw->setSnapRatio(f);

  Py_INCREF(Py_None); return Py_None;
}

PyObject* getsnapstate(PyObject *self, PyObject *args) {

  if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }

  return Py_BuildValue("i", gw->snapState());
}

// Actuator

// FIXME: don't know if it works (Doesn't work)
// don't know if is it needed either. KFandangoWidget redraws every
// timerEvent signal, and is probably threaded. Making it a NO_OP by now.
PyObject* update(PyObject *self, PyObject *args) {

  if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }

  //  gw->update_screen();
  
  Py_INCREF(Py_None); return Py_None;
}

PyObject* ratio(PyObject *self, PyObject *args){

  if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }

  return Py_BuildValue("f", gw->getRatio());
}

PyObject* clicks(PyObject *self, PyObject *args){

  if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }
  float const * const fp = gw->getClickQueue();
  return Py_BuildValue("[[fff][fff][fff]]", 
		       fp[3], fp[4], fp[5],
		       fp[6], fp[7], fp[8],
		       fp[9], fp[10], fp[11]);
}

PyObject* snaps(PyObject *self, PyObject *args){

  if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }
  float const * const fp = gw->getSnapQueue();
  return Py_BuildValue("[[fff][fff][fff]]", 
		       fp[3], fp[4], fp[5],
		       fp[6], fp[7], fp[8],
		       fp[9], fp[10], fp[11]);
}

PyObject* find_entity(PyObject *self, PyObject *args){
  float x, y, z = 0.f;

  if (!PyArg_ParseTuple(args, "ff|f", &x, &y, &z)){
    return NULL;
  }

  return Py_BuildValue("i", gw->findByCoordinates(x, y, z));
}

PyObject* set_selected(PyObject *self, PyObject *args){

  int i;

  if (!PyArg_ParseTuple(args, "i", &i)){
    return NULL;
  }

  gw->setSelected(i);

  Py_INCREF(Py_None); return Py_None;
}


PyObject* get_errors(PyObject *self, PyObject *args){

  if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }

  gw->reportErrors();

  Py_INCREF(Py_None); return Py_None;
}


PyObject* get_selected(PyObject *self, PyObject *args){

  if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }

  return Py_BuildValue("i", gw->getSelected());
}

PyObject* get_intersection(PyObject *self, PyObject *args){

  if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }

  float const * const fp = gw->getLastIntersection();

  return Py_BuildValue("[fff]", fp[0], fp[1], fp[2]);
}

PyObject* move(PyObject *self, PyObject *args) {

  float x, y;
  float z = 0.f;

  if (!PyArg_ParseTuple(args, "ff|f", &x, &y, &z)){
    return NULL;
  }

  gw->moveVertex(x, y, z);
  
  Py_INCREF(Py_None); return Py_None;
}

PyObject* viewport_zoom(PyObject *self, PyObject *args){
  float l, t, b;
  l = -260.f; t = 200.f; b = -200.f;
  if (!PyArg_ParseTuple(args, "|fff", &l, &t, &b)){
    return NULL;
  }
  
  gw->setViewportCorners(l, t, b);
  Py_INCREF(Py_None); return(Py_None);
}

PyObject* rotation_axis(PyObject *self, PyObject *args){
  float lx, ly, lz;
  lx = 0.f; ly = 1.f; lz = 0.f;
  if (!PyArg_ParseTuple(args, "|fff", &lx, &ly, &lz)){
    return NULL;
  }
  
  if(lx || ly || lz) { // check that the vector is not zero
    gw->setRotationAxis(lx, ly, lz);
  }

  Py_INCREF(Py_None); return(Py_None);
}

PyObject* clear(PyObject *self, PyObject *args){
  if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }
  gw->clear();
  
  Py_INCREF(Py_None); return(Py_None);
}

PyObject* reset(PyObject *self, PyObject *args){
  if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }
  gw->resetTransformations();
  
  Py_INCREF(Py_None); return(Py_None);
}

PyObject* lock(PyObject *self, PyObject *args){

  int b;
  if (!PyArg_ParseTuple(args, "i", &b)){
    return NULL;
  }
  gw->lock(b);
  
  Py_INCREF(Py_None); return(Py_None);
}

PyObject* rot_style(PyObject *self, PyObject *args){

  int b;
  if (!PyArg_ParseTuple(args, "i", &b)){
    return NULL;
  }
  gw->setOldRotation(b);
  
  Py_INCREF(Py_None); return(Py_None);
}

// Very unefficient; blocks a very long time to recover a full page
// and turn it on a tuple. (tested on 12k points pages)
PyObject* get_entity(PyObject *self, PyObject *args){

  int i;

  if (!PyArg_ParseTuple(args, "i", &i)){
    return NULL;
  }

  GLenum type;
  int size;

  // FIXME: check for error
  // Verify too that i is a valid entity
  float const * const fp = gw->getEntity(i, type, size);

  PyObject *ret, *ret2;

  // Construct an empty tuple
  ret2 = PyTuple_New(size);

  // And fill it
  for(int j = 0; j < size; ++j){
    ret = Py_BuildValue("(fff)", 
			*(fp + (3*j) + 0),  
			*(fp + (3*j) + 1), 
			*(fp + (3*j) + 2));
    PyTuple_SetItem(ret2, j, ret);
  }

  return Py_BuildValue("(iO)", type, ret2);

}

PyObject* null_placeholder_function(PyObject *self, PyObject *args){

  if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }

  Py_INCREF(Py_None); return Py_None;
}

PyObject* datapath(PyObject *self, PyObject *args){

  if (!PyArg_ParseTuple(args, "")){
    return NULL;
  }

  return Py_BuildValue("s", DATADIR);
}

}
