/* 
 *    python-cad, a GL rendered, python scripted drawing engine
 *                  Copyright (C) 2002, 2003 by
 *
 *      Jaime Soffer             (jsoffer@soffernet.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef PRIMITIVES_H
#define PRIMITIVES_H 

#include <Python.h>

extern "C" {

PyObject* stdout_write(PyObject *self, PyObject *args);
PyObject* stderr_write(PyObject *self, PyObject *args);
PyObject* bg(PyObject *self, PyObject *args);

void initStdout(void);
void initStderr(void);
void initGLState(void);
void initDrawing(void);
void initGet(void);
void initEdit(void);
void initInput(void);

// stack operations

PyObject* line(PyObject *self, PyObject *args);
PyObject* triangle(PyObject *self, PyObject *args);
PyObject* line_from(PyObject *self, PyObject *args, PyObject *keywds);
PyObject* line_to(PyObject *self, PyObject *args);
PyObject* tris_from(PyObject *self, PyObject *args);
PyObject* tris_to(PyObject *self, PyObject *args);
PyObject* beginTrianglesEntity(PyObject *self, PyObject *args);
PyObject* beginLinesEntity(PyObject *self, PyObject *args);

// GL state wrappers
PyObject* set_line_width(PyObject *self, PyObject *args);
PyObject* set_color(PyObject *self, PyObject *args);
PyObject* set_light_color(PyObject *self, PyObject *args);
PyObject* set_light_position(PyObject *self, PyObject *args);
//PyObject* set_clear_color(PyObject *self, PyObject *args); // now bg
PyObject* line_antialiasing(PyObject *self, PyObject *args);
PyObject* blending(PyObject *self, PyObject *args);
PyObject* texture(PyObject *self, PyObject *args);
PyObject* lighting(PyObject *self, PyObject *args);
PyObject* highlight(PyObject *self, PyObject *args);
PyObject* cull(PyObject *self, PyObject *args);

// Accessors
PyObject* getsnap(PyObject *self, PyObject *args);
PyObject* findint(PyObject *self, PyObject *args);
PyObject* getint(PyObject *self, PyObject *args);
PyObject* getsnapvertex(PyObject *self, PyObject *args);
PyObject* setsnap(PyObject *self, PyObject *args);
PyObject* setsnapratio(PyObject *self, PyObject *args);
PyObject* getsnapstate(PyObject *self, PyObject *args);
PyObject* get_entity(PyObject *self, PyObject *args);
PyObject* get_errors(PyObject *self, PyObject *args);

PyObject* ratio(PyObject *self, PyObject *args);
PyObject* clicks(PyObject *self, PyObject *args);
PyObject* snaps(PyObject *self, PyObject *args);
PyObject* find_entity(PyObject *self, PyObject *args);
PyObject* set_selected(PyObject *self, PyObject *args);
PyObject* get_selected(PyObject *self, PyObject *args);
PyObject* get_intersection(PyObject *self, PyObject *args);
PyObject* rotation_axis(PyObject *self, PyObject *args);
PyObject* rot_style(PyObject *self, PyObject *args);

PyObject* lock(PyObject *self, PyObject *args);

// Actuators

PyObject* clear(PyObject *self, PyObject *args);
PyObject* reset(PyObject *self, PyObject *args);
PyObject* update(PyObject *self, PyObject *args);
PyObject* move(PyObject *self, PyObject *args);
PyObject* viewport_zoom(PyObject *self, PyObject *args);


PyObject* null_placeholder_function(PyObject *self, PyObject *args);

PyObject* datapath(PyObject *self, PyObject *args);

}

#endif
