/* 
 *    fandango, a GL rendered, python scripted drawing engine
 *                  Copyright (C) 2002, 2003 by
 *
 *      Jaime Soffer             (jsoffer@soffernet.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "qpyobject.hpp"

QPyObject::QPyObject(int argc, char** argv,
		     QObject* parent, char* name)
  : QObject(parent, name){

  Py_Initialize();
  
  char* foo = argv[0];
  PySys_SetArgv(0, &foo); // Stupid hack to make me feel falsely safe
  // The full story: vaguely I remember, some time ago, a python error
  // (can't remember which) when PySys_SetArgv hasn't been called. But
  // lately (20030526) I found out that setting KCmdLineArgs::init AND
  // calling kfandango from the minicli segfaults here, leading me to 
  // think these factors modify somehow the given values.

  initStdout();
  initStderr();
  initGLState();
  initDrawing();
  initGet();
  initEdit();
  initInput();

  // Match if only alphanumeric and spaces 
  // (fixed: added minus sign and point)
  isSimpleRxp = QRegExp( "^[\\w\\s-\\.]+$", FALSE, FALSE );
  printRxp = QRegExp( "^\\s*print", TRUE, FALSE );
}

QPyObject::~QPyObject(void){
  Py_Finalize();
}

void QPyObject::processExternalCommand(const QString & qs){

  if (qs.contains(isSimpleRxp) && !qs.contains(printRxp)) {

    // The python command is in "simple" format: only alphanumeric,
    // no symbols, just one command
    // globals() is to export the context to the wrapper
    QString simple_command = 
      "cli_wrapper.simple_eval(\"" + 
      qs + 
      "\", globals())";
    PyRun_SimpleString((char *) simple_command.latin1());

  } else {
    // Run the command through standard Python
    PyRun_SimpleString((char *) qs.latin1());
  }

}

// TODO: port to the python unicode facilities
void QPyObject::initialize(bool dotfile){
  
  QString path = DATADIR;
  path += "/python";

  PyRun_SimpleString("import sys; import os");

  QString qs = "sys.path.append('" + path + "')";
  PyRun_SimpleString((char *) qs.latin1());

  QString exec_init = "execfile('" + path + "/fandango.py')";
  PyRun_SimpleString((char *) exec_init.latin1());

  if(dotfile){
    PyRun_SimpleString("if(os.path.isfile(os.path.expanduser('~/.fandango'))): \
 execfile(os.path.expanduser('~/.fandango'))");
  }
}

#include "qpyobject.moc"
