/* 
 *    fandango, a GL rendered, python scripted drawing engine
 *                  Copyright (C) 2002, 2003 by
 *
 *      Jaime Soffer             (jsoffer@soffernet.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef QPYOBJECT_H
#define QPYOBJECT_H

#include <Python.h>

#include <qobject.h>
#include <qregexp.h>

#include "primitives.hpp"

/**
 * It initializes and closes the python interpreter, and receives QStrings
 * to convert into char* and execute using PyRun_SimpleString.
 */
class QPyObject : public QObject
{
  Q_OBJECT

  private:
  
  /**
   * Used to determine if the input was a Python command or a simplified string
   * A simplified string contains just alphanumeric characters and spaces
   */
  QRegExp isSimpleRxp;

  /**
   * Used to determine if the string is a 'print' statement, since them can't
   * be eval'ed so easily. (FIXME: chain with previous regexp)
   */
  QRegExp printRxp;

  public:

  /**
   * Runs Py_Initialize(); PySys_SetArgv(argc, argv); and initializes
   * all the extended python functions.
   */
  QPyObject(int, char**, QObject*, char*);

  /**
   * Runs Py_Finalize.
   */
  virtual ~QPyObject(void);

  /**
   * Imports sys, os, appends the data directory to the path,
   * imports the base modules and then executes the dotfile.
   */
  void initialize(bool dotfile);

  public slots:

  /**
   * Executes PyRun_SimpleString(arg); this is the central python
   * interpreter of the application.
   */
  void processExternalCommand(const QString &);

};

#endif
